{# Site footer. #}
<footer class="background-midnight-blue color-white">
  <div class="container">
    <div class="row">

      <div class="col-md-4">

        <h3>Info</h3>

        <ul class="nav-footer">
          <li>Item</li>
          <li>Item</li>
          <li>Item</li>
          <li>Item</li>
        </ul>

        <p>
          <small class="text-muted">
              &copy; 2017 Websauna team
          </small>
        </p>
      </div>

      <div class="col-md-4">
        <h3>Moar footer</h3>

        <ul class="nav-footer">
          <li>Item</li>
          <li>Item</li>
          <li>Item</li>
        </ul>
      </div>
      <div class="col-md-4">
        <h3>Follow</h3>
        {% include "newsletter/subscription_form.html" %}
      </div>
    </div>

</footer>